/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

// @(#) $Id: IDomRow.java 2089 2007-11-28 13:56:13Z s3460 $
package at.spardat.xma.datasource;

import at.spardat.xma.mdl.Atom;

/**
 * Models a row in a {@link ITabularDomData}.
 * 
 * @author YSD, 20.06.2003 19:38:54
 */
public interface IDomRow {

    /**
     * The value of the column <tt>COD_KEY</tt> in the <tt>ITabularDomData</tt>. 
     * The returned value is never <tt>null</tt> and unique amongst the rows
     * in the <tt>ITabularDomData</tt>.
     */
    public String getKey ();
    
    /**
     * The value of the column <tt>SHORT_VALUE</tt> in the <tt>ITabularDomData</tt>.
     * The returned value is never <tt>null</tt> but may be empty String.
     */
    public String getShortValue ();
    
    /**
     * The value of the column <tt>LONG_VALUE</tt> in the <tt>ITabularDomData</tt>.
     * The returned value is never <tt>null</tt> but may be empty String.
     */
    public String getLongValue ();
    
    /**
     * Returns true if the rows <tt>VALID_FROM</tt> and <tt>VALID_TO</tt> time range
     * includes the current time, i.e., returns true if the current time is not
     * before <tt>VALID_FROM, 0:00</tt> and not after <tt>VALID_TO, 23:59:59.999</tt>. 
     */
    public boolean isInValidTimeRange ();
    
    /**
     * Returns the value of this row for the column with a given column-index. <p>
     * 
     * Normally, this method is not needed, since you access the rows cells via
     * the other get-methods defined in this class. However, if your <tt>ITableProvider</tt>
     * wants to provide more columns than required for a <tt>ITabularDomData</tt>, 
     * the auxiliary cells may be accessed via this method. 
     * 
     * @param columnIndex  zero-based index of the column. 
     * @return non <tt>null</tt> Atom.
     * @exception RuntimeException if <tt>columnIndex</tt> is greater than
     *             or equal to the number of columns in the table.
     */
    public Atom getCell (int columnIndex);
}
