/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 13.06.2003
 *
 * 
 * 
 */
package at.spardat.xma.security;

import javax.security.auth.Subject;
import javax.servlet.http.HttpSession;

/**
 * This is the interface of all server side authentication plugins.
 * Server side authentication plugins are used to check the user data
 * send by the client. Server side authentication plugins cooperate with
 * their client side authenticatin plugins (JAAS LoginModule). The
 * LoginModule usually encrypts the credentials in an way the server
 * side authentication module can verify; e.g. by using a challenge
 * response procedure. 
 * 
 * 
 * @author s2877
 */
public interface LoginModuleServer {
    
    /**
     * Get information needed on the client side before the login. 
     * Eg. some defaults for the login dialog or a challenge for a challange response procedure.
     * @param session the HttpSession 
     * @return an object containing this information. This object must be serializable.
     */
    Object getPreLoginInfo(HttpSession session);
    
    /**
     * Do the login on the server. This typically 
     * validates the principals and credentials of the given subject and add some
     * more information to the principal.
     * @param subject the subject as returned by the corresponding LoginModule
     *      Any changes in the subject are send back to the client by the runtime.
     * @param session the HttpSession 
     * @return true if the subject is successfully verified
     *      false otherwise
     */
    boolean login(Subject subject, HttpSession session);
    
    /**
     * Do the logout on the server. This may release some resources.
     * @param subject the subject as returned by the corresponding LoginModule
     * @param session the HttpSession
     * @return true if the logout was successfull
     */
    boolean logout(Subject subject, HttpSession session);
}
